
%{

function G_new = adfilterNormConvWithConfMeasure(G,Var_inv, n, sigma_target, h_start, h_inc, h_max)

Denoises an image G with adaptive normalized convolution. The kernel size
is adapted based on an estimate of the inverse variance Var_inv.

The method is described in 
Mario Frank, Matthias Plaue and Fred A. Hamprecht
"Denoising of continuous-wave time-of-flight depth images using confidence measures".
In Optical Engineering (Vol. 48, No. 7), SPIE 2009.

Please contact us if you find bugs or ways to improve the code.
Mario Frank, 2011


Input: 
    G       :   Original noisy image, m*n matrix
    Var_inv :   confidence measure, should ideally be an estimate of the 
                inverse variance per pixel in G.  m*n matrix
    n       :   mask size 'n', and the desired variance (optional input)
    sigma_target : desired variance of denoised pixels (optional input)
    h_start :   smallest kernel width of Gaussian kernel for normalized 
                convolution (optional input)
    h_inc   :   kernel width increment (optional input)
    h_max   :   maximal kernel width


Output: 
    G_new   :   a filtered image, m*n matrix

%}


function G_new = adfilterNormConvWithConfMeasure(G,Var_inv, n, sigma_target, h_start, h_inc, h_max)


%% apply filter with all possible kernel widths
G_hs(:,:,1)   = G;                  %initialise array of images with original image
Var_hs(:,:,1) = 2*sigma_target*ones(size(G));    %initialise array of variance with values higher than sigma_target,
for h = h_start : h_inc : h_max
    
    a = gaussvek(n,h);
    nom    = conv2( a.^2,a.^2, Var_inv , 'same');    %conv( 1/var , gauss(h)^2) ; nominator  
    denom  = conv2( a,a, Var_inv , 'same');          %conv( 1/var , gauss(h)  ) ; denominator 
    
    Var_hs = cat( 3, Var_hs, nom ./ (denom.^2) );    % array of new variances of 2D-data as a function of h. Is eq. to conv(sqrt(var),gauss)/denom^2  
    G_hs   = cat( 3, G_hs  , conv2(a,a, G .*(Var_inv) , 'same') ./ denom  );  % array of smoothed 2D-data as a function of h. Is eq. to conv( G/var ,gauss(h))/denom 
        
end
counter = size (G_hs,3);  %number of smoothed images in G_hs     


%% determine filter strength
d= [1 -1];
Steps =  double( Var_hs(:,:,:) > sigma_target );
selection = double ( filter(d,1,Steps,[],3) < 0 );   % <0 prevents to detect steps in other direction
selection(:,:,counter) = selection(:,:,counter) +1 -sum(selection,3) ;  % solves problem if nowhere is a 1 -> then highest h is taken; 


%% copy filtered pixels to new image
G_new = sum (selection .*G_hs, 3);

end



function F = gaussvek(n,h)
% creates a gaussian

k = linspace(-(n-1)/2,(n-1)/2,n);
F = 1./exp(k.^2/(2*h^2));
F= F /h /sqrt(2*pi) ;
end













